(******************************************************************************

  Modeling and Verifying Properties of Security Protocols in Wireless Networks

  Module: distance/MessageDerivation.thy (Isabelle/HOL 2009)

  Copyright (C) 2007-2009 Benedikt Schmidt <benedikt.schmidt@inf.ethz.ch>
  License: BSD (see COPYRIGHT)

******************************************************************************)

header {* Derivation of Messages *}

theory MessageDerivation imports Public begin

subsection {* Derivation of Nonces *}

lemma (in INITSTATE_NONONCE) othernonce_gen_received:
  assumes A: "Nonce B NB \<in> subterms {X}" and ineq: "A\<noteq>B" and
          B: "X \<in> DM A (knowsI A tr)"
  shows   "\<exists> t i Y. (t, Recv (Rx A i) Y) \<in> set tr \<and> Nonce B NB \<in> subterms {Y}"
  using A B ineq
  apply -
  apply (subgoal_tac "Nonce B NB \<in> subterms (knowsI A tr)")
  prefer 2
  apply (rule_tac A=A in nonce_subterms_DM_nonce)
  apply (subgoal_tac "{X} \<subseteq> DM A (knowsI A tr)")
  apply (drule subterms.mono)
  apply (erule subsetD)
  apply force
  apply force
  apply force
  apply (drule subterms.singleton) back
  apply (auto)
  apply (drule knowsI_A_imp_Recv_initState)
  apply (erule disjE)
  apply auto
  apply (drule_tac H="initState A" and G="{Y}" in subterms.trans)
  apply force
  apply (insert no_nonce_initState_subterms, auto)
done


lemma (in INITSTATE_NONONCE_PARTS) othernonce_gen_received_parts:
  assumes A: "Nonce B NB \<in> parts {X}" and ineq: "A\<noteq>B" and
          B: "X \<in> DM A (knowsI A tr)"
  shows   "\<exists> t i Y. (t, Recv (Rx A i) Y) \<in> set tr \<and> Nonce B NB \<in> parts {Y}"
  using A B ineq
  apply -
  apply (subgoal_tac "Nonce B NB \<in> parts (knowsI A tr)")
  prefer 2
  apply (rule_tac A=A in nonce_parts_DM_nonce)
  apply (subgoal_tac "{X} \<subseteq> DM A (knowsI A tr)")
  apply (drule parts.mono)
  apply (erule subsetD)
  apply force
  apply force
  apply force
  apply (drule parts.singleton) back
  apply (auto)
  apply (drule knowsI_A_imp_Recv_initState)
  apply (erule disjE)
  apply auto
  apply (drule_tac H="initState A" and G="{Y}" in parts.trans)
  apply force
  apply (insert no_nonce_initState_parts, auto)
done

subsection {* Derivation of Signatures *}

context INITSTATE_PKSIG begin

lemma sig_knowsI_sig_received:
  assumes A: "X \<in> knowsI A tr" and AnotB: "A \<noteq> (Honest B)" and
          B: "Crypt (priSK (Honest B)) msig \<in> subterms {X}"
  shows "\<exists>t i. (t, Recv (Rx A i) X) \<in> set tr"
 using A B AnotB
  apply -
  apply (drule knowsI_A_imp_Recv_initState)
  apply (erule disjE)
  apply auto
  apply (drule_tac H="initState A" and G="{X}" in subterms.trans)
  apply force
  apply (insert priSK_not_used, auto)
done

end

end