(******************************************************************************

  Modeling and Verifying Properties of Security Protocols in Wireless Networks

  Module: distance/Misc.thy (Isabelle/HOL 2009)

  Copyright (C) 2007-2009 Benedikt Schmidt <benedikt.schmidt@inf.ethz.ch>
  License: BSD (see COPYRIGHT)

******************************************************************************)

header {* Some general lemmas needed in the formalization *}

theory Misc imports Main Real begin

lemma Un_snd [simp]: "fst`(UN x. H x) = (UN x. fst`(H x))"
by (auto)

lemma app_union [simp]: "f`(X  \<union> Y) = (f`X \<union>  f`Y)"
by (auto)

lemma app_bUnion [simp]:
  "f`(\<Union>x\<in>H. G x) = (\<Union>x\<in>H. f`(G x))"
by auto


(*Needed occasionally with spy_analz_tac, e.g. in analz_insert_Key_newK*)
lemma [simp] : "A \<union> (B \<union> A) = B \<union> A"
by blast

text{*By default only @{text o_apply} is built-in.  But in the presence of
eta-expansion this means that some terms displayed as @{term "f o g"} will be
rewritten, and others will not!*}
declare o_def [simp]

lemma fst_set[simp]: "fst`{ev. ev = (a,b,c) \<and> P} = {m. m = a \<and> P}"
by auto

lemma subsetD2: "\<lbrakk>c \<in> A; A \<subseteq> B\<rbrakk> \<Longrightarrow> c \<in> B"
by auto 

lemma set_un_eq: "[| A = B; C = D |] ==> A \<union> C = B \<union> D"
by auto

section {* Agents, Key distributions, and Transceivers *}
types
  key = nat
  time = real

consts
  invKey        :: "key=>key"  --{*inverse of a symmetric key*}

specification (invKey)
  invKey [simp]: "invKey (invKey K) = K"
    by (rule exI [of _ id], auto)

datatype  --{*We allow any number of honest agents and intruders *}
  agent = Honest nat | Intruder nat

instantiation agent :: linorder
begin

fun
  less_agent :: "agent \<Rightarrow> agent \<Rightarrow> bool"
 where
  "(Honest a)   < (Honest b)   = (a < b)" |
  "(Honest a)   < (Intruder b) = True" |
  "(Intruder b) < (Honest a)   = False" |
  "(Intruder a) < (Intruder b) = (a < b)"

definition
  less_eq_agent:  "(a::agent) \<le> b = ((a = b) \<or> (a < b))"

instance proof
  fix x y :: agent show "(x < y) = (x \<le> y \<and> \<not> y \<le> x)"
    apply (auto simp add: less_eq_agent)
    apply (case_tac x, auto)
    apply (case_tac x)
    apply (case_tac y, auto)+
    done
next
  fix x :: agent show "x \<le> x" by (auto simp add: less_eq_agent)
next
  fix x y z ::agent show "\<lbrakk>x \<le> y; y \<le> z\<rbrakk> \<Longrightarrow> x \<le> z"
    apply (auto simp add: less_eq_agent)
    apply (case_tac x)
      apply (case_tac y)
        apply (case_tac z)
	apply auto
      apply (case_tac z)
      apply auto
    apply (case_tac y, auto)
    apply (case_tac z, auto)
    done
next
  fix x y :: agent show "\<lbrakk>x \<le> y; y \<le> x\<rbrakk> \<Longrightarrow> x = y"
    apply (auto simp add: less_eq_agent)
    apply (case_tac x)
      apply (case_tac y, auto)
    apply (case_tac y, auto)
    done
next
  fix x y :: agent show "x \<le> y \<or> y \<le> x"
    apply (auto simp add: less_eq_agent)
    apply (case_tac x, case_tac y, auto, case_tac y, auto)
    done
qed

end

datatype
  transmitter = Tx agent nat
 
datatype
  receiver = Rx agent nat

lemmas [split] = transmitter.split receiver.split
                 transmitter.split_asm receiver.split_asm

end