(******************************************************************************

  Modeling and Verifying Properties of Security Protocols in Wireless Networks

  Module: distance/SystemCoffset.thy (Isabelle/HOL 2009)

  Copyright (C) 2007-2009 Benedikt Schmidt <benedikt.schmidt@inf.ethz.ch>
  License: BSD (see COPYRIGHT)

******************************************************************************)

header {* Systems with constant local-clock Offsets *}

theory SystemCoffset imports SystemSimps SystemOrigination begin

consts
  coffset :: "friendid \<Rightarrow> time"

specification (clocktime)
  clocktime_coff[simp] : "clocktime A t = t + coffset A"
  apply auto
  done

locale PROTOCOL_DELTAONLY = PROTOCOL +
  assumes proto_time_delta:
    "step \<in> proto \<Longrightarrow>
     (step  (timetrans A tr) A (clocktime A t)) =
     (step tr A t)"

lemma (in PROTOCOL_DELTAONLY) view_timetrans1:
  assumes a:
   "(\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step (timetrans A tr) A (clocktime A t)\<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))"
  shows
   "(\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step tr A t \<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))"
proof -
  fix tr t step m pEv A
  assume "step \<in> proto" and "(m,pEv) \<in> step tr A t" and
         "tr \<in> sys_param" and "P tr" and "t >=  maxtime tr"
  thus "P ((t, createEv A pEv m) # tr)" apply -
    apply (rule a [where t="t" and A=A, simplified])
    apply force prefer 3
    apply assumption
    apply (auto simp add: proto_time_delta [simplified])
    done
qed

lemma (in PROTOCOL_DELTAONLY) view_timetrans2:
  assumes a:
   "(\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step tr A t \<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))"
  shows
   "(\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step (timetrans A tr) A (clocktime A t) \<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))"
proof -
  fix step m t pEv A tr
  assume "step \<in> proto" and "(m,pEv) \<in> step (timetrans A tr) A (clocktime A t)" and
         "tr \<in> sys_param" and "P tr" and "t>=  maxtime tr"
  thus "P ((t,createEv A pEv m)#tr)" apply -
    apply (rule a [where t="t" and A=A, simplified])
    apply (auto simp add: proto_time_delta [simplified])
    done
qed


lemma (in PROTOCOL_DELTAONLY) timetrans_removable:
 "(\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t;
      step \<in> proto; (m,pEv) \<in> step (timetrans A tr) A (clocktime A t) \<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))
  ==
  (\<And>tr t step m pEv A.
    \<lbrakk> tr \<in> sys_param; P tr; maxtime tr <= t; step \<in> proto; (m,pEv) \<in> step tr A t\<rbrakk>
    \<Longrightarrow> P ((t,createEv A pEv m)#tr))"
  apply (rule Pure.equal_intr_rule)
  apply (rule view_timetrans1)
  apply auto
  apply (rule view_timetrans2)
  apply auto
done

locale PROTOCOL_DELTA_EXEC = PROTOCOL_DELTAONLY + PROTOCOL_EXECUTABLE

text {* These two only hold if PROTOCOL\_EXECUTABLE is instantiated with sys,
        e.g. the first equality holds *}

lemma (in PROTOCOL_DELTA_EXEC) sys_Proto_exec:
  "\<lbrakk> sys = sys_param; tr \<in> sys_param; maxtime tr \<le> t;
     step \<in> proto; (m, pEv) \<in> step (timetrans A tr) A (clocktime A t)\<rbrakk>
   \<Longrightarrow> (t, createEv A pEv m) # tr \<in> sys"
  apply (rule sys.Proto)
  apply force
  apply force apply force
  apply (subst events_occur_at)
  apply force apply force apply force
  apply (rule messages_derivableI)
  apply force
  apply (subst events_occur_at)
  apply auto
done

lemma (in PROTOCOL_DELTA_EXEC) sys_Proto:
  "\<lbrakk>sys = sys_param; step \<in> proto; (m, pEv) \<in> step tr A t;
    tr \<in> sys_param; maxtime tr \<le> t\<rbrakk>
   \<Longrightarrow> (t, createEv A pEv m) # tr \<in> sys"
  apply (subgoal_tac "(t, createEv A pEv m) # tr \<in> sys")
  apply force
  apply (rule sys_Proto_exec)
  apply force
  apply force defer
  apply force
  apply (simp only: proto_time_delta)
  apply force
done

lemma in_timetrans:
  "((t,e) \<in> set (timetrans A tr)) = ((t - coffset A, e) \<in> set tr)"
  apply (auto simp add: timetrans_def intro!: bexI)
done

end